from datetime import datetime
from sceptre.hooks import Hook
import boto3
import subprocess
import json

from typing import List


class CustomHook(Hook):
    def __init__(self, *args, **kwargs):
        super(CustomHook, self).__init__(*args, **kwargs)
        self._lambda = boto3.session.Session().client("lambda", region_name="eu-central-1")

    def run(self):
        """
        run is the method called by Sceptre. It should carry out the work
        intended by this hook.
        """
        # self.argument == "deploy_start" || "deploy_end"
        try:
            self.lambda_handler(self.argument)
        except:
            # just ignore any error for now
            pass

    def lambda_handler(self, method: str) -> None:
        payload = {
            "method": method,
            "git_commit_message": self._get_last_git_commit_message(),
            "git_branch_name": self._get_git_branch_name(),
            "stack_name": self.stack.name,
            "time": datetime.utcnow().isoformat()
        }
        resp = self._lambda.invoke(
            FunctionName="arn:aws:lambda:eu-central-1:521248050649:function:sceptre-lifecycle-provider-vpc-cad5d5a2",
            InvocationType="RequestResponse", Payload=json.dumps(payload))
        print(resp.read())

    def _get_last_git_commit_message(self) -> str:
        return self._get_output_subprocess(["git", "log", "-1"])

    def _get_git_branch_name(self) -> str:
        return self._get_output_subprocess(["git", "branch", "--show-current"])

    @staticmethod
    def _get_output_subprocess(command: List[str]) -> str:
        return subprocess.run(command, capture_output=True).stdout.decode("utf-8")



import sys


if sys.version_info < (3,7):
    sys.exit('Sorry, Python < 3.7 is not supported')

from setuptools import find_packages
from distutils.core import setup

setup(
    name='sceptre-deployment-hook',  # How you named your package folder (MyLib)
    packages=find_packages(exclude=['test']),
    version="0.0.10",
    license='Apache-2.0',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Sceptre hook to measure stack deployment time',  # Give a short description about your library
    author='Vereniging COIN',  # Type in your name
    author_email='devops@coin.nl',  # Type in your E-Mail
    url='https://gitlab.com/verenigingcoin/open-source/sceptre-hooks/deployment-hook',
    # Provide either the link to your github or to your website
    keywords=['COIN', 'AWS', 'Sceptre'],  # Keywords that define your package best
    python_requires='>=3.7',
    zip_safe=False,
    platforms='any',
    entry_points={
        'sceptre.hooks': [
            'stack_deploy_time = hook.hook:CustomHook',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        # Choose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3.7',
    ],
)
